@@ -271,6 +271,9 @@ def calc_regression_criterion(
 
     criterion_method = kwargs.get("criterion_method", "mse_norm")
 
+    if isinstance(y, pd.Series):
+        y = y.values
+
     if criterion_method.lower() == "mse":
         criterion = mse_function(prediction, y)
     elif criterion_method.lower() == "mse_norm":
@@ -613,3 +616,13 @@ def calc_ruleset_prediction_equally_weighted_classificator_stacked(prediction_ve
         most_freq_pred = pd.Series([most_freq_pred])
     most_freq_pred.name = None
     return most_freq_pred.reindex(idx)
+
+
+def calc_zscore_external(
+    prediction: Union[pd.Series, float], nones: Union[pd.Series, int], y: np.ndarray, horizon: int = 1
+) -> Union[None, float, pd.Series]:
+    if isinstance(nones, int) and nones == 0:
+        return np.nan
+    num = abs(prediction - np.nanmean(y))
+    deno = np.sqrt(horizon / nones) * np.nanstd(y)
+    return num / deno
